#pragma once
#include <string>
#include <vector>
#include <iostream>
#include <fstream>
#include <algorithm>
#include <functional>
using namespace std;

// A function to trim whitespace from both sides of a given wstring
inline void Trim(std::wstring& str, const std::wstring & ChrsToTrim = L" \t\n\r", int TrimDir = 0)
{
    size_t startIndex = str.find_first_not_of(ChrsToTrim);
    if (startIndex == std::wstring::npos){str.erase(); return;}
    if (TrimDir < 2) str = str.substr(startIndex, str.size()-startIndex);
    if (TrimDir!=1) str = str.substr(0, str.find_last_not_of(ChrsToTrim) + 1);
}


class CIniFile
{
public:
	struct Record
	{
		wstring Comments;
		wchar_t Commented;
		wstring Section;
		wstring Key;
		wstring Value;
	};

	enum CommentChar
	{
		Pound = '#',
		SemiColon = ';'
	};

	CIniFile(void);
	virtual ~CIniFile(void);

	static bool AddSection(wstring SectionName, wstring FileName);
	static bool CommentRecord(CommentChar cc, wstring KeyName,wstring SectionName,wstring FileName);
	static bool CommentSection(char CommentChar, wstring SectionName, wstring FileName);
	static wstring Content(wstring FileName);
	static bool Create(wstring FileName);
	static bool DeleteRecord(wstring KeyName, wstring SectionName, wstring FileName);
	static bool DeleteSection(wstring SectionName, wstring FileName);
	static vector<Record> GetRecord(wstring KeyName, wstring SectionName, wstring FileName);
	static vector<Record> GetSection(wstring SectionName, wstring FileName);
	static vector<wstring> GetSectionNames(wstring FileName);
	static wstring GetValue(wstring KeyName, wstring SectionName, wstring FileName);
	static bool RecordExists(wstring KeyName, wstring SectionName, wstring FileName);
	static bool RenameSection(wstring OldSectionName, wstring NewSectionName, wstring FileName);
	static bool SectionExists(wstring SectionName, wstring FileName);
	static bool SetRecordComments(wstring Comments, wstring KeyName, wstring SectionName, wstring FileName);
	static bool SetSectionComments(wstring Comments, wstring SectionName, wstring FileName);
	static bool SetValue(wstring KeyName, wstring Value, wstring SectionName, wstring FileName);
	static bool Sort(wstring FileName, bool Descending);
	static bool UnCommentRecord(wstring KeyName,wstring SectionName,wstring FileName);
	static bool UnCommentSection(wstring SectionName, wstring FileName);

private:
	static vector<Record> GetSections(wstring FileName);
	static bool Load(wstring FileName, vector<Record>& content);	
	static bool Save(wstring FileName, vector<Record>& content);

	struct RecordSectionIs : std::unary_function<Record, bool>
	{
		std::wstring section_;

		RecordSectionIs(const std::wstring& section): section_(section){}

		bool operator()( const Record& rec ) const
		{
			return rec.Section == section_;
		}
	};

	struct RecordSectionKeyIs : std::unary_function<Record, bool>
	{
		std::wstring section_;
		std::wstring key_;

		RecordSectionKeyIs(const std::wstring& section, const std::wstring& key): section_(section),key_(key){}

		bool operator()( const Record& rec ) const
		{
			std::wstring s(rec.Section.c_str()), k(rec.Key.c_str());
			std::wstring s1(section_.c_str()), k1(key_.c_str());
			std::transform(s.begin(), s.end(), s.begin(), tolower);
			std::transform(k.begin(), k.end(), k.begin(), tolower);
			std::transform(s1.begin(), s1.end(), s1.begin(), tolower);
			std::transform(k1.begin(), k1.end(), k1.begin(), tolower);
			Trim(s);
			Trim(s1);
			Trim(k);
			Trim(k1);

			return ((s == s1)&&(k == k1));
		}
	};

	struct AscendingSectionSort
	{
		bool operator()(Record& Start, Record& End)
		{
			return Start.Section < End.Section;
		}
	};

	struct DescendingSectionSort
	{
		bool operator()(Record& Start, Record& End)
		{
			return Start.Section > End.Section;
		}
	};

	struct AscendingRecordSort
	{
		bool operator()(Record& Start, Record& End)
		{
			return Start.Key < End.Key;
		}
	};

	struct DescendingRecordSort
	{
		bool operator()(Record& Start, Record& End)
		{
			return Start.Key > End.Key;
		}
	};
};